@@ -5,7 +5,7 @@
 /*
  * print queue memory management
  * Copyright Nicholas B. Tufillaro, 1982-1994. All rights reserved.
- * GNU enhancements copyright (C) 1996-1997 Free Software Foundation, Inc.
+ * GNU enhancements copyright (C) 1996-1999 Free Software Foundation, Inc.
  */
 
 struct prt *
@@ -35,6 +35,6 @@ pfree (pp)
   if (pp != NULL) 
     {
       pfree (pp->pr_link);
-      free ((Voidptr)pp);
+      free ((voidptr_t)pp);
     }
 }
